use strict;
use Test::More 0.98;

use_ok $_ for qw(
    Getopt::EX::termcolor
    Getopt::EX::termcolor::Apple_Terminal
    Getopt::EX::termcolor::iTerm
    Getopt::EX::termcolor::XTerm
);

done_testing;
